% PARAFAC Tutorial: using the drEEM and N-way toolboxes for MATLAB
% This file shows example code for implementing PARAFAC analysis, and is
% intended to accompany the tutorial in Appendix A of the reference below
%
% Notice:
% This mfile is part of the drEEM toolbox. Please cite the toolbox
% as follows:
%
% Murphy K.R., Stedmon C.A., Graeber D. and R. Bro, Fluorescence
%     spectroscopy and multi-way techniques. PARAFAC, Anal. Methods, 2013, 
%     DOI:10.1039/c3ay41160e. 
%
% Copyright (C) 2013 Kathleen R. Murphy
% The University of New South Wales
% Dept Civil and Environmental Engineering
% Water Research Center
% UNSW 2052
% Sydney
% krm@unsw.edu.au
%
% $ Version 0.1.0 $ September 2013 $ First Release

%% Set up toolboxes and demo on MATLAB path (edit paths as required)
%Toolboxes
addpath('C:/Program Files/MATLAB/R2012b/extra_toolboxes/drEEM/nway') %edit for correct file destination
addpath('C:/Program Files/MATLAB/R2012b/extra_toolboxes/drEEM/')     %edit for correct file destination

%Demo - put a copy of the demo dataset in a writeable folder, e.g.
demopath='C:/MyDATAdirectory/MyMATLABdirectory/PARAFAC tutorial/drEEM/demo'; %edit for correct file destination

%Get started by loading the data in the demo folder
load([demopath '/Corrected_EEMs/Backup/PortSurveyData_corrected.mat'])

%Overview of toolbox
help drEEM

%lookforconflicts('DOMfluor')
lookforconflicts('drEEM')
lookforconflicts('nway')

%% Assemble dataset, smooth EEM scatter, and remove outliers

%Assemble dataset
mydata= assembledataset( XcRU_df1,Ex,Em_in,'QSE','site',sites,'rep',replicates,'longID',filelist_eem,'ID',sampleID,'cruise',cruises,'date',dates,[]);
eemview(mydata,'X',[3 3])
classinfo(mydata)

%Remove noisy data then smooth the EEM
SubData=subdataset(mydata,[],mydata.Em<270,mydata.Ex<250);
eemview(SubData,'X',[3 3],1,[],'SampleID')

%Xs=smootheem(SubData,[15 15],[15 15],[17 18],[19 18],[0 1 1 0],[],3500,'pause'); %check plots   ###can see the image
Xs=smootheem(SubData,[15 15],[15 15],[17 18],[19 18],[0 1 1 0],[],3500,0);%complete the smoothing procedure without plotting

%View EEMs and remove unusual samples
Xin=Xs
eemview(Xs,'X',[3 3],1,[],'SampleID')
%Xin=subdataset(Xs,{'site','bl','0A'},[],[]);

%Eliminate faulty parts of EEMs
%hit Cntrl-C to cancel eemview
eemview(Xin,'X',[1 3],176,[],[],[],[],[],20)
Xin=zap(Xin,176,[],[345 350])
eemview(Xin,'X',[1 3],176,[],[],[],[],[],20)
Xin=zap(Xin,1:205,[560 576],[280 295])
eemview(Xin,'X',[1 3],[],[],[],[],[],[],20)

%clean up workspace
clear XcQS XcRU1 Ex Em_in cruises sites replicates dates sampleID filelist_eem
close all

%% Exploratory Phase
    
%Rapid search for outliers: Test1
Test1=outliertest(Xin,[2,2],1:8,'nonnegativity',[],'at once');
Test1=outliertest(Xin,[2,2],4,'nonnegativity',[],'at once');
Test1=outliertest(Xin,[2,2],5,'nonnegativity',[],'at once');
Test1=outliertest(Xin,[2,2],4:5,'nonnegativity',[],'at once');
Test1=outliertest(Xin,[2,2],4:6,'nonnegativity',[],'at once');
Test1u=outliertest(Xin,[2,2],3:7,'unconstrained',[],'at once');

%Visualise the spectra
comparespectra(Test1,1:8)
comparespectra(Test1,4:6)
comparespectra(Test1,4:5)
comparespectra(Test1,3:7)
comparespectra(Test1u,3:7)

%Are components highly correlated? If so, normalise the data.
compcorrplot(Test1,8)
compcorrplot(Test1,4)
compcorrplot(Test1,5)
compcorrplot(Test1,6)

%Limiting the effect of dilution or highly correlated components
%by normalising intensities to unit norm
Xpre=normeem(Xin) %normalise samples to unit norm
Test1p=outliertest(Xpre,[1,1],1:4,'nonnegativity',[],'at once');
Test2p=outliertest(Xpre,[1,1],5:8,'nonnegativity',[],'at once');

Test1p=outliertest(Xpre,[1,1],5,'nonnegativity',[],'at once');
Test1p=outliertest(Xpre,[1,1],4:5,'nonnegativity',[],'at once');
spectralloadings(Test1p,5:7)

%View correlations in the normalised dataset
compcorrplot(Test2p,5)
compcorrplot(Test1p,4)
compcorrplot(Test1p,4)
compcorrplot(Test1p,5)
compcorrplot(Test1p,6)

%Look for outliers
loadingsandleverages(Test1p,5) 
loadingsandleverages(Test1p,6)  
loadingsandleverages(Test1p,7) 

%Remove outliers and redo outlier test
% but first, recreate the normalised dataset observing the correct order
% of preprocessing steps, where normalisation always comes last
Xin=subdataset(Xin,[205,208],[],[]);
Xpre=normeem(Xin)
Test2=outliertest(Xpre,[2,2],5:7,'nonnegativity',[],'at once');
loadingsandleverages(Test2,5) 
loadingsandleverages(Test2,6)  
loadingsandleverages(Test2,7)
spectralloadings(Test2,5:7)

%Examine residuals for outliertest Test2, especially for outliers
eemview({Test2, Test2},{'X','Model6','error_residuals'},[1 3],49)

%compare residuals for outliertest Models 5 and 6, both in Test2
compare2models(Test2,5,6,0.05)

%save expphase.mat Xs Xin Xin2 Xpre Test1 Test2 Test1u Test2p Test2r
%load expphase

%% Validation Phase
%Refine the models
% Obtain least squares models 
[LSmodel1,convg1,DSit1]=randinitanal(Xpre,1,5,'nonnegativity',1e-8);%%%five means five runs here
[LSmodel2,convg2,DSit2]=randinitanal(Xpre,2,5,'nonnegativity',1e-8);
[LSmodel3,convg3,DSit3]=randinitanal(Xpre,3,5,'nonnegativity',1e-8);
[LSmodel4,convg4,DSit4]=randinitanal(Xpre,4,5,'nonnegativity',1e-8);

[LSmodel5,convg5,DSit5]=randinitanal(Xpre,5,5,'nonnegativity',1e-8);
[LSmodel6,convg6,DSit6]=randinitanal(Xpre,6,5,'nonnegativity',1e-8);
[LSmodel7,convg7,DSit7]=randinitanal(Xpre,7,5,'nonnegativity',1e-8);
[LSmodel8,convg8,DSit8]=randinitanal(Xpre,8,5,'nonnegativity',1e-8);

% Visualize model loadings, e.g. for the 6-comp. model
spectralloadings(LSmodel1,1)
spectralloadings(LSmodel2,2)
spectralloadings(LSmodel3,3)
spectralloadings(LSmodel4,4)
spectralloadings(LSmodel5,5)
spectralloadings(LSmodel6,6)
spectralloadings(LSmodel7,7)
spectralloadings(LSmodel8,8)


spectralloadings(LSmodel4,4)
comparespectra(LSmodel4,4)
spectralloadings(LSmodel5,5)
comparespectra(LSmodel5,5)
fingerprint(LSmodel5,5)
spectralloadings(LSmodel6,6)
comparespectra(LSmodel6,6)
fingerprint(LSmodel6,6)
fingerprint(LSmodel4,4)
fingerprint(LSmodel5,5)


% Reverse the normalisation step to recover the true scores
% The reversed model is exactly the same as the original, except that the
% scores are no longer normalised.
LSmodel1r=normeem(LSmodel1,'reverse',1)
LSmodel2r=normeem(LSmodel2,'reverse',2)
LSmodel3r=normeem(LSmodel3,'reverse',3)
LSmodel4r=normeem(LSmodel4,'reverse',4)

LSmodel5r=normeem(LSmodel5,'reverse',5)
LSmodel6r=normeem(LSmodel6,'reverse',6)
LSmodel7r=normeem(LSmodel7,'reverse',7)
LSmodel8r=normeem(LSmodel8,'reverse',8)

%compare the unnormalised and normalised plots
compcorrplot(LSmodel6,6,[],'cruise')
compcorrplot(LSmodel6r,6,[],'cruise')

%Validation by Split Half Analysis
%Split and combine the dataset (S4C6)
S1=splitds(Xpre,[],4,'alternating',{[1 2],[3 4],[1 3],[2 4],[1 4],[2 3]}); 
%Create PARAFAC models in each split of S1
A1=splitanalysis(S1,1,'nonnegativity',[],[],'A1');
A1=splitanalysis(S1,2,'nonnegativity',[],[],'A1');
A1=splitanalysis(S1,3,'nonnegativity',[],[],'A1');
A1=splitanalysis(S1,4,'nonnegativity',[],[],'A1');

A1=splitanalysis(S1,5,'nonnegativity',[],[],'A1');
A1=splitanalysis(S1,4:5,'nonnegativity',[],[],'A1');
A1=splitanalysis(S1,5:6,'nonnegativity',[],[],'A1');

A1=splitanalysis(S1,6,'nonnegativity',[],[],'A1');
A1=splitanalysis(S1,7,'nonnegativity',[],[],'A1');
A1=splitanalysis(S1,8,'nonnegativity',[],[],'A1');

%Preliminary attempt to validate 5- and 6-component models
splitvalidation(A1,1,[1 2;3 4;5 6]);
splitvalidation(A1,2,[1 2;3 4;5 6]);
splitvalidation(A1,3,[1 2;3 4;5 6]);
splitvalidation(A1,4,[1 2;3 4;5 6]);

splitvalidation(A1,5,[1 2;3 4;5 6]);
splitvalidation(A1,6,[1 2;3 4;5 6]);
splitvalidation(A1,7,[1 2;3 4;5 6]);
splitvalidation(A1,8,[1 2;3 4;5 6]);

%Improve the 6-component split models by taking the best of multiple runs
%and/or decrease the convergence criterion
%6-component model
MyRunOptions=[3 3 3 3 3 3] %specify runs for each split individually
MyCC=1e-6
SaveResultsAs='A2_10-6_6comp'
A2b=splitanalysis(S1,6,'nonnegativity',MyRunOptions,MyCC,SaveResultsAs);
%5-component model
%Increase runs, decrease convergence criterion
MyRunOptions=4 %4 runs for each split
MyCC=1e-8
SaveResultsAs='A2_10-8_5comp'
A2a=splitanalysis(S1,5,'nonnegativity',MyRunOptions,MyCC,SaveResultsAs);

%Complete the  validation of 5 component model in 3 tests (S4C6T3)
%Use LSmodel5r (containing the unscaled scores)
val5=splitvalidation(A2a,5,[1 2;3 4;5 6],{'AB','CD','AC','BD','AD','BC'},LSmodel5r);
val4=splitvalidation(A2a,4,[1 2;3 4;5 6],{'AB','CD','AC','BD','AD','BC'},LSmodel4r);
spectralloadings(val4,4,1:4)
spectralloadings(val5,5,1:5)

%Complete the validation of 6 component model in 3 tests (S4C6T3)
%Use LSmodel6r (containing the unscaled scores)
val6=splitvalidation(A2b,6,[1 2;3 4;5 6],{'AB','CD','AC','BD','AD','BC'},LSmodel6r);
spectralloadings(val6,6,1:6)

%Examine error residuals for Model 6
eemview({LSmodel6,LSmodel6},{'X','Model6','error_residuals'},[2 3]);
loadingsandleverages(DSit6,6,5); %Observe leverages on model in e.g. Run 5

%Compare error residuals for least squares models 5 and 6
eemview({LSmodel5,LSmodel5,LSmodel6,LSmodel6},{'X','Model5','error_residuals','X','Model6','error_residuals'},[2 3],[],[],[],[1 1 0.05 1 1 0.05]);

%Can compare 3 or more models/datasets using eemview, as long as they depict the 
%exact same set of samples
%For example, plot EEMs, model and residuals for the 4-6 component models
%in Test 1:
MyDataSources={Test1,Test1,Test1,Test1,Test1,Test1}; %Specify where the raw data and model come from for each Modeln
MyData2Plot={'X','Model4','error_residuals','X','Model5','error_residuals','X','Model6','error_residuals'}; %residuals are difference between the preceding two datasets.
MyPlotTitles={'Test1','M4','error','Test1','M5','error','Test1','M6','error'}; %Label plots in figures
MyAxisScale=[1 1 0.05, 1 1 0.05, 1 1 0.05]; %Set error residuals to 5% of the model and raw data
MyAxisOrientation='rotate'; %put Ex on the horizontal axis
MyContourDensity=20; %override MATLAB default number of contours
MyColorbar=[]; %alternative: 'colorbar'
eemview(MyDataSources,MyData2Plot,[3 3],[],[],MyPlotTitles,MyAxisScale,MyAxisOrientation,MyColorbar,MyContourDensity)
close all

%save valphase.mat S1 A1 A2a A2b Xin2 LSmodel5 LSmodel6 LSmodel7 LSmodel6p LSmodel7p val5 val6
%save valphase.mat LSmodel6p LSmodel7p  -append
%load valphase

%% View some results prior to exporting

%view Fmax for a particular model

[F5,scores5]=scores2fmax(val5,5)
[F6,scores6]=scores2fmax(val6,6)                               %#ok<*ASGLU>

%Describe components in terms of Ex and Em maxima
summary=describecomp(val6,6);
summary=describecomp(val6,5);

%% Export data
%Basic model export, including converting to Fmax
modelout(val5,5,'drEEM_PARAFAC_demoX.xls');
modelout(val6,6,'drEEM_PARAFAC_demoX.xls');
modeloutmac(val5,5,'drEEM_PARAFAC_demoX.txt');

%Export metadata along with a model
metadata4export={'cruise','site','rep','longID','ID','date','i'}
modelout(val6,6,'drEEM_PARAFAC_demo.xls',[],metadata4export);

%Project the full dataset (Xs) on to the model in val6 to get scores for outliers
[F6,EmSpectra,ExSpectra,Ff,P6]=modelout(val6,6,'drEEM_PARAFAC_demo.xls',Xs);

%Look at the model fit for the projected dataset
eemview({P6,P6},{'X','Model5_P','error_residuals'},[3 3],[],[],[],[1 1 0.05]);
close all

%Export to openfluor
 openfluor(LSmodel5, 5, 'Model5.txt')
 openfluor(LSmodel6, 6, 'C:/Data/MATLAB/PARAFAC/Model6.txt')

%% Incorporate hypothesis testing into split half analysis
%Below are some different ways to split the dataset 
%1. Split by cruise 
bycruisedata=splitds(Xpre,'cruise',[],'exact'); %split by cruise
%check Splits contain desired sample combinations:
for i=1:4; disp(bycruisedata.Split(i).cruise);pause(1);end 

%2. Split by cruise then combine into pairs of cruises
bycruisedata=splitds(Xpre,'cruise',[],'exact'); %split by cruise
cruisecomb=splitds(bycruisedata,[],[],'combine',{[1 2],[3 4],[1 3],[2 4],[1 4],[2 3]}); 
%check Splits contain desired sample combinations:
for i=1:6; i,disp(cruisecomb.Split(i).cruise);pause(1);end 

%2. Split by alternating method, keeping replicate samples together
%First split by cruise.site then combine into a alternating S4C6T3 
bycruisesite=splitds(Xpre,'cruise.site',[],'exact');
S=bycruisesite.Split_NumBeforeCombine;
scomb={[1:4:S 2:4:S],[3:4:S 4:4:S],[1:4:S 3:4:S],[2:4:S 4:4:S],[1:4:S 4:4:S],[2:4:S 3:4:S]};
cruisesitecomb=splitds(bycruisesite,[],[],'combine',scomb);
%check Splits contain desired sample combinations
for i=1:6
    [cruisesitecomb.Split(i).cruise cruisesitecomb.Split(i).site cruisesitecomb.Split(i).rep]
    pause(1);
end
%% When a model does not validate
%Example: Try to validate a 4-component by-cruise model, with each split
% containing a different cruise with different sample sizes.
bc4=splitanalysis(bycruisedata,4,'nonnegativity',5,[],'bc4');
%components can plot in different orders
spectralloadings(bc4,4,1:4); %plot 4-comp models for 4 splits 
%try to validate the models by comparing each possible pair of cruises
vbc4=splitvalidation(cc4,4); %(6 tests)
%Check which components found matches
r4=relcomporder(vbc4,1) %reorders components relative to split model 1.
%fill in the NaNs by trial and error
r4(2,1)=3;r4(2,3)=1;r4(3,1)=2;r4(3,3)=1;r4(4,1)=2;r4(4,3)=1;
%replot while reordering the components
spectralloadings(bycruise4,4,1:4,r4); 

%Example: Try to validate a 6-component cruise-combination model
spectralloadings(vcc6,6,1:6); %components can plot in different orders
allcc6=splitvalidation(cc6,6); %ignore/close these plots
r=relcomporder(allcc6,1) %determine component order relative to Split 1 (of 6)
r(2,6)=6; r(3,6)=5; r(4,6)=5; r(6,6)=5;  %fill in NaNs
spectralloadings(allcc6,6,1:6,r)  %plot relative to Split 1

%plot the components
allcs6=splitvalidation(cs6,6); %ignore plots
r1=relcomporder(allcs6,1) %#ok<*NOPTS> %determine component order relative to Split 1 (of 6)
spectralloadings(allcs6,6,1:6,r1)  %plot relative to Split 1

%Working with split models
 %create a dataset from Split 1
Split1mod=cc6.Split(1);
 %look for outliers
loadingsandleverages(Split1mod,6); 
loadingsandleverages(vcc6.Split(4),6); 
 %residuals for split models
eemview({Split1mod, Split1mod},{'X','Model6','error_residuals'},[1 3],99);
 %remove sample 99 from Split1mod
Split1mod_R=subdataset(Split1mod,99,[],[]);
 %what was sample 99 in the original dataset?
Split1mod.i(99)
 %get a least squares model of Split 1
LS_Split1mod_R=randinitanal(Split1mod_R,6,5,'nonnegativity',1e-8);
 %look for outliers again....and so on!
loadingsandleverages(LS_Split1mod_R,6); %look for outliers again....

%TIPS:
%remember that if you find and remove outlier data you must go back and 
%recreate your dataset making normalisation the last thing that you do!
%this is especially important after using 'zap'!


% Notice:
% This mfile is part of the drEEM toolbox. Please cite the toolbox
% as follows:
%
% Murphy K.R., Stedmon C.A., Graeber D. and R. Bro, Fluorescence
%     spectroscopy and multi-way techniques. PARAFAC, Anal. Methods, 2013, 
%     DOI:10.1039/c3ay41160e. 
%
% scores2fmax: Copyright (C) 2013 Kathleen R. Murphy
% The University of New South Wales
% Dept Civil and Environmental Engineering
% Water Research Center
% UNSW 2052
% Sydney
% krm@unsw.edu.au
%
% $ Version 0.1.0 $ September 2013 $ First Release